$NOMOD51
$NOLIST
$include(reg51.inc)
$TITLE(robert-stadler@gmx.de - Midi Controller POD 2.0)

;Midi Controller Futasterleiste speziell auf Line6 POD 2.0 angepasst
;--------------------------------------------------------------------
;Specs
;
;- ber DIP-Fix Schalter einstellbarer Midi-Kanal (gilt fr alle Signale)
;- 9 frei programmierbare Futaster (ber diesen Code!)
;	HIER: 	+ 4 absolute Programme (#1 bis #4)
;		+ 3 Controller Ereignisse (Delay, Boost, Distortion)
;		+ Programme aufwrts und abwrts anwhlen



;Variablendefinitionen im Bitadressierbaren Bereich
channel DATA 20h	;Midi Kanal durch DIP-Fix schalter eingestellt, einlesen bei initialisierung
program DATA 21h	;Speicher fr das zuletzt ausgewhlte Midi-Programm
flanke1 DATA 22h	;Flankenmerker Port 1
flanke3 DATA 23h	;Flankenmerker Port 3
ausein1	DATA 24h	;Ein-Aus Flip-Flop Speicher fr Port 1
ausein3	DATA 25h	;Ein-Aus Flip-Flop Speicher fr Port 3


org 00h
ljmp init


org  100h
init:
;Konfiguration serielle Schnittstelle 31250 baud
	mov pcon,#00h 		;Taktverdoppler aus
	mov tmod,#00100010B  	;timer 1 und 2 als 8-bit auto-reload counter 
	mov th1,#254    	;preset wert
	mov tl1,#254    	;ladeschacht
	setb tcon.6     	;startet timer1
	mov scon,#01000010B     ;setzt seriellen modus mit 8 bit  und TI bit

;Ports als Eingnge vorbereiten
	mov P3,#0FFh
	mov P1,#0FFh

;Midi Channel am DIP-Fix Schalter auslesen und speichern
	mov a,P3
	rr  a
	rr  a
	cpl a
	anl a,#0Fh
	mov channel,a

;Variablen initialisieren
	mov program,#0
	mov flanke1,#0
	mov flanke3,#0
	mov ausein1,#0
	mov ausein3,#0

;Stack-Pointer in unbemnutzten Bereich zeigen lassen
	mov sp,#30h

;Anfang Hauptschleife ----------------------------------------

abfrage0:
	jnb  p1.0, bit10z		;Falls Taster gedrckt, Sprung nach bit10z
	jnb flanke1.0, abfrage1		;Falls Taster nicht gedrckt und Flankenmerker nicht gesetzt, Sprung nach abfrage1
	clr flanke1.0			;Falls Taster nicht gedrckt, flankenmerker lschen
	acall zeit			;Entprellen
	ljmp abfrage1			;weiter
bit10z: jb flanke1.0, abfrage1		;weiter wenn flankenmerker gesetzt -> Signal wurde schon gesendet
	setb flanke1.0			;Flankenmerker setzen
	ljmp prog1			;Signal senden

abfrage1:
	jb  p1.1, abfrage2
	ljmp bankup

abfrage2:
	jnb  p1.2, bit12z
	jnb flanke1.2, abfrage3
	clr flanke1.2
	acall zeit
	ljmp abfrage3
bit12z: jb flanke1.2, abfrage3
	setb flanke1.2
	ljmp prog2

abfrage3:
	jb  p1.3, abfrage4	
	ljmp bankdown

abfrage4:
	jnb  p1.4, bit14z	
	jnb flanke1.4, abfrage5
	clr flanke1.4
	acall zeit
	ljmp abfrage5
bit14z: jb flanke1.4, abfrage5
	setb flanke1.4
	ljmp prog3

abfrage5:
	jnb  p1.5, bit15z
	jnb flanke1.5, abfrage6
	clr flanke1.5
	acall zeit
	ljmp abfrage6
bit15z: jb flanke1.5, abfrage6
	setb flanke1.5	
;	ljmp tuner
	ljmp effekt3

abfrage6:
	jnb  p1.6, bit16z
	jnb flanke1.6, abfrage7
	clr flanke1.6
	acall zeit
	ljmp abfrage7
bit16z: jb flanke1.6, abfrage7
	setb flanke1.6	
	ljmp prog4

abfrage7:			;Flankenauswertung
	jnb p1.7, bit17z 	;0=bettigt ->Sprung zur Bearbeitung
	jnb flanke1.7,abfrage8	;nicht bettigt, Flankenmerker schon gelscht
	clr flanke1.7		;nicht bettigt, Flankenmerker lschen
	acall zeit		;Entprellung
	ljmp abfrage8		;nchste Taste
bit17z:	jb flanke1.7, abfrage8	;Falls nich erster Zyklus -> nchste Taste
	setb flanke1.7		;Flankenmerker setzen
	ljmp effekt1

abfrage8:
	jnb p3.7, bit37z 	
	jnb flanke3.7,abfrage9
	clr flanke3.7
	acall zeit
	ljmp abfrage9	
bit37z:	jb  flanke3.7, abfrage9
	setb flanke3.7
	ljmp effekt2

abfrage9:
	ljmp abfrage0	

;-------------------------------------------Ende Hauptschleife

	
;Funktionen =============================================

prog1:
	mov a,#0C0h  		;Program-Change
	add a,channel
	acall sendbyte
	mov a,#1
	mov program,a
	acall sendbyte		
	acall zeit
	ljmp abfrage1


prog2:
	mov a,#0C0h  		;Program-Change
	add a,channel
	acall sendbyte
	mov a,#2
	mov program,a
	acall sendbyte		
	acall zeit
	ljmp abfrage3

prog3:
	mov a,#0C0h  		;Program-Change
	add a,channel
	acall sendbyte
	mov a,#3
	mov program,a
	acall sendbyte		
	acall zeit
	ljmp abfrage5

prog4:
	mov a,#0C0h  		;Program-Change
	add a,channel
	acall sendbyte
	mov a,#4
	mov program,a
	acall sendbyte		
	acall zeit
	ljmp abfrage7

;tuner:
;	mov a,#0C0h  		;Controller-Change
;	add a,channel
;	acall sendbyte
;	mov a,#37
;	mov program,a
;	acall sendbyte		
;	acall zeit
;	ljmp abfrage6

effekt3:	
	mov a,#0B0h  		;Controller-Change
	add a,channel
	acall sendbyte
	mov a,#25		;Controller-#25 = Distortion On/Off
	acall sendbyte
	jb ausein1.5, effekt3aus
	setb ausein1.5		;Zustand merken
	mov a,#127
	acall sendbyte
	acall zeit
	ljmp abfrage6
effekt3aus:
	clr ausein1.5
	clr a
	acall sendbyte
	acall zeit
	ljmp abfrage6



;Bank durchdrcken aufwrts
bankup:
	mov a,#0C0h  		;Nchster Kanal
	add a,channel
	acall sendbyte
	mov a,program
	inc a
	cjne a,#38,bankup1
	mov a,#0
bankup1:
	mov program,a
	acall sendbyte		
	acall wzeit
	ljmp abfrage2

;Bank durchdrcken abwrts
bankdown:
	mov a,#0C0h  		;Vorheriger Kanal
	add a,channel
	acall sendbyte
	mov a,program
	dec a
	cjne a,#255,bankdown1
	mov a,#37
bankdown1:	
	mov program,a
	acall sendbyte		
	acall wzeit
	ljmp abfrage4

effekt1:	
	mov a,#0B0h  		;Controller-Change
	add a,channel
	acall sendbyte
	mov a,#28		;Controller-#28 = Delay On/Off
	acall sendbyte
	jb ausein1.7, effekt1aus
	setb ausein1.7		;Zustand merken
	mov a,#127
	acall sendbyte
	acall zeit
	ljmp abfrage8
effekt1aus:
	clr ausein1.7
	clr a
	acall sendbyte
	acall zeit
	ljmp abfrage8

effekt2:
	mov a,#0B0h  		;Controller-Change
	add a,channel
	acall sendbyte
	mov a,#26		;Controller-#26 = Drive/Boost On/Off
	acall sendbyte
	jb ausein3.7, effekt2aus
	setb ausein3.7
	mov a,#127
	acall sendbyte
	acall zeit
	ljmp abfrage9
effekt2aus:
	clr ausein3.7
	clr a
	acall sendbyte
	acall zeit
	ljmp abfrage9


;100ms Wartezeit
Zeit:	push 00h	;R0
	push 01h	;R1
	push 02h	;R2
	mov r0,#2
Zeit1:	mov r1,#200
Zeit2:	mov r2,#250	
Zeit3:	djnz r2,Zeit3
	djnz r1,Zeit2
	djnz r0,Zeit1
	pop 02h
	pop 01h
	pop 00h
	ret
	
;300ms Wartezeit
WZeit:	push 00h	;R0
	push 01h	;R1
	push 02h	;R2
	mov r0,#6
WZeit1:	mov r1,#200
WZeit2:	mov r2,#250	
WZeit3:	djnz r2,WZeit3
	djnz r1,WZeit2
	djnz r0,WZeit1
	pop 02h
	pop 01h
	pop 00h
	ret
	
	
;Byte ber TxD ausgeben
;bergabe des Byte in a
sendbyte:
        jnb scon.1,sendbyte	;solange noch nicht fertig gesendet, warten
        clr scon.1		;fertig gesendet -> lschen
        mov sbuf,a		;neues Bytet senden
        ret


end
